;+
; NAME:
;  skysclim
; PURPOSE:
;  Compute stretch range for a hard stretch on the background in an image.
; DESCRIPTION:
; CATEGORY:
;  Image display
; CALLING SEQUENCE:
;  skysclim,image,lowval,hival,meanval,sigma
; INPUTS:
;  image - 2-d image to compute stretch range for.
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
;  NPTS - Maximum number of points to grab at random from image for scaling
;           information.  Default=601.
;  LOWCLIP - fraction of random sample to clip at the low end of the signal.
;              If lowclip=.1 and npts=100, then the 10 lowest values in the
;              random sample are excluded BEFORE the robust mean is computed
;              for the stretch range.  This option will probably be just a bit
;              slower if invoked.  This option will likely be more robust against
;              extreme values in the image.  Default=0.0 (no clipping)
;  HICLIP  - fraction of random sample to clip at the high end of the signal.
;              If hiclip=.9 and npts=100, then the 10 highest values in the
;              random sample are excluded BEFORE the robust mean is computed
;              for the stretch range.  This option will probably be just a bit
;              slower if invoked.  This option will likely be more robust against
;              extreme values in the image.  Default=1.0 (no clipping)
; OUTPUTS:
;  lowval - Low DN value for sky stretch  (meanval-3.0*sigma)
;  hival  - High DN value for sky stretch (meanval+5.0*sigma)
;  meanval - Mean of random sample
;  sigma   - standard deviation of random sample
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  96/01/07 - Marc W. Buie
;  2000/02/29, MWB, added NPTS keyword
;  2002/11/17, MWB, added LOWCLIP, HICLIP keywords
;  2002/12/18, MWB, fixed bug causing crash when image is smaller than npts.
;-
pro skysclim,image,lowval,hival,meanval,sigma, $
   NPTS=npts,LOWCLIP=lowclip,HICLIP=hiclip

   if badpar(npts,[0,2,3],0,caller='SKYSCLIM: (NPTS) ',default=601) then return
   if badpar(lowclip,[0,4,5],0,caller='SKYSCLIM: (LOWCLIP) ',default=0.0) then return
   if badpar(hiclip,[0,4,5],0,caller='SKYSCLIM: (HICLIP) ',default=1.0) then return

   lowclip=lowclip > 0.0
   hiclip = hiclip < 1.0

   cnpts = min([npts,n_elements(image)])

   ; generate random indicies to get a random sample of the image
   idx=randomu(seed,cnpts)*(n_elements(image)-1)

   if lowclip gt 0.0 or hiclip lt 1.0 then begin

      ; extract portion
      timage = image[idx]
      ; sort
      s = sort( timage )

      ; get index bounds of clipped region
      t1 = long(lowclip * cnpts) > 0L
      t2 = long(hiclip * cnpts) < long(cnpts-1)

      robomean,timage[s[t1:t2]],3.0,0.5,meanval,dummy,sigma

   endif else begin

      robomean,image[idx],2.0,0.5,meanval,dummy,sigma

   endelse

   lowval=meanval-3.0*sigma
   hival=meanval+5.0*sigma

end
