;+
; NAME:
;   struct_equal
;
; PURPOSE:
;   function allows to compare two structures variables
;
;
; CALLING SEQUENCE:
;   Result =  struct_equal(struct1,struct2, COMPARE=Tags)
;
;EXAMPLE:
;    pro EX_structcompare
;
;    afile = FILEPATH('image.tif', SUBDIRECTORY = ['examples', 'data'])
;    bfile = FILEPATH('examples.tif', SUBDIRECTORY = ['examples', 'data'])
;
;    Result = QUERY_TIFF(afile, aInf)
;    Result = QUERY_TIFF(bfile, bInf)
;
;    aInfo = {IMAGE: dist(4)*2, $
;          R: 0, $
;          G: 0, $
;          B: 0, $
;          QUERY:aInf $
;           }
;
;    bInfo = {IMAGE: dist(4), $
;          R: 0, $
;          G: 0, $
;          B: 1, $
;          QUERY:bInf $
;           }
;
;    q = struct_equal(aInfo, bInfo, COMPARE=tags)
;    if q eq -1 then begin
;      q = dialog_message('not comparable structure')
;      return
;    end
;
;    for i=0, n_elements(tags)-1 do begin
;      cmd1 = "print, 'ainfo'+tags[i]+' =',"+'ainfo'+tags[i]
;      excmd1 = execute(cmd1)
;      cmd2 = "print, 'binfo'+tags[i]+' =',"+'binfo'+tags[i]
;      excmd2 = execute(cmd2)
;      print,''
;    endfor
;    end
;
; RESULT:  -1 : not comparable structures
;           0 : comparable different structures
;           1 : equal structures
;
; KEYWORDS:
;   COMPARE: returns the tags names where structures are diffents
;
;
; MODIFICATION HISTORY:
;   Written by:  Stefano Gagliano, RSI, 12-2007
;   Modified:
;
;-


function struct_equal, str1, str2, COMPARE=compare

  sname1 = tag_names(str1,/STRUCTURE_NAME)
  sname2 = tag_names(str2,/STRUCTURE_NAME)

  compare = 'not comparable'
  if sname1 ne sname2 then return,-1 ;// not comparable

;// anounymous structure
  if (sname1 eq '') then begin
    ntag1 = n_tags(str1)
    ntag2 = n_tags(str2)
    if ntag1 ne ntag2 then return,-1 ;// not comparable
    tnames1 = tag_names(str1)
    tnames2 = tag_names(str2)
    tag_compare = strcmp(tnames1,tnames2)
    if total(tag_compare) ne ntag1 then return,-1 ;// not comparable
  endif else begin
    ntag1 = n_tags(str1)
    tnames1 = tag_names(str1)
  endelse

  count = 0
  pos = ''
  for i=0, ntag1-1 do begin
    if size(str1.(i),/type) eq 8 then begin
      q = struct_equal(str1.(i), str2.(i), COMPARE=tags)
      if q eq 0 then begin
        pos = [pos,'.'+tnames1[i]+tags]
        count++
      endif
    endif else begin
      if ~array_equal(str1.(i),str2.(i)) then begin
        pos = [pos,'.'+tnames1[i]]
        count++
      endif
    endelse
  endfor

  if count eq 0 then begin
    compare = pos
    return, 1
  endif else begin
    compare = pos[1:*]
    return, 0
  endelse

end