; Finds the Nearest Value in Vector with a maximum allowed error
; Similar to IDL function value_locate() except for:
;     1) This function finds nearest neighbor
;     2) Sets output to -1 if error between Value and Nearest Neighbor exceeds maximum
; @param vector[] : input vector, used to locate nearest neighbor
; @param value[]  : for each element of value[], find nearest neighbor in vector[]
; @returns array the size of value[]
; TODO: what to do when there is more than one 1 equivalent match?  E.g. nearest_value( [1,2,3,4,5,6], 5.5 )
function neighbor_locate, vector, value, ERROR_MAX=error_max

    n_values = n_elements( value )
    nearest = make_array( n_values, /L64, value=-1 )
    
    for i_value = 0, n_values - 1 do begin

        ; IEEE NaN        
        if ( ~ finite( value[ i_value ] ) ) then begin
            nearest[ i_value ] = -1
            continue
        endif

        error = abs(vector - value[ i_value ])
        error_min = min( error, i_min )

        if ( n_elements( error_max ) gt 0 ) then begin
            if ( error_min gt error_max ) then begin
                nearest[ i_value ] = -1
            endif $
            else begin
                nearest[ i_value ] = i_min
            endelse
        endif $
        else begin
            nearest[ i_value ] = i_min
        endelse
    endfor
    
    ; for scalar input => scalar output 
    if ( n_values eq 1 ) then nearest = nearest[0]

    return, nearest
    
end


function test_neighbor_locate

    retval = 1

    ; Test1: scalar input, expected scalar output
    neighbor = neighbor_locate( [1,2,3,4,5,6], 3.3, ERROR_MAX=1 )
    result = ( neighbor eq 2 and size( neighbor, /DIMENSIONS ) eq 0 ) ? "true" : "false"
    if ( result eq "false" ) then retval = 0
    print, "Test1 => ", result


    ; Test2: value array input, expected array output
    neighbors = neighbor_locate( [1,2,3,4,5,6], [-1, 2.4, 2.51, 6, 6.1, 7.1], ERROR_MAX=1 )
    result = -1 eq total( where( neighbors eq [ -1, 1, 2, 5, 5, -1 ] ne 1 ) ) ? "true" : "false"
    if ( result eq "false" ) then retval = 0
    print, "Test2 => ", result


    ; Test3: value array input with NaN entry, expected array output with -1 at the NaN entry
    neighbors = neighbor_locate( [1,2,3,4,5,6], [-1, !values.f_nan, 2.4, 2.51, 6, 6.1, 7.1], ERROR_MAX=1 )
    result = -1 eq total( where( neighbors eq [ -1, -1, 1, 2, 5, 5, -1 ] ne 1 ) ) ? "true" : "false"
    if ( result eq "false" ) then retval = 0
    print, "Test3 => ", result


    ; return
    return, retval
end
